=== modified file '.bzrignore'
--- .bzrignore	2009-05-23 04:55:52 +0000
+++ .bzrignore	2009-06-03 14:14:31 +0000
@@ -38,6 +38,7 @@
 ./api
 doc/**/*.html
 doc/developers/performance.png
+bzrlib/_bencode_pyx.c
 bzrlib/_btree_serializer_c.c
 bzrlib/_chk_map_pyx.c
 bzrlib/_chunks_to_lines_pyx.c

=== modified file 'NEWS'
--- NEWS	2009-06-03 06:43:45 +0000
+++ NEWS	2009-06-05 07:14:08 +0000
@@ -20,6 +20,14 @@
 New Features
 ************
 
+* ``--development7-rich-root`` is a new dev format, similar to ``--dev6``
+  but using a Revision serializer using bencode rather than XML.
+  (Jelmer Vernooij, John Arbash Meinel)
+
+* mail_client=claws now supports --body (and message body hooks).  Also uses
+  configured from address.  (Barry Warsaw)
+
+
 Improvements
 ************
 
@@ -28,6 +36,12 @@
   bugs with stacking and non default formats.)
   (John Arbash Meinel, #373455)
 
+* ``--development6-rich-root`` delays generating a delta index for the
+  first object inserted into a group. This has a beneficial impact on
+  ``bzr commit`` since each committed texts goes to its own group. For
+  committing a 90MB file, it drops peak memory by about 200MB, and speeds
+  up commit from 7s => 4s. (John Arbash Meinel)
+
 * Numerous operations are now faster for huge projects, i.e. those
   with a large number of files and/or a large number of revisions,
   particularly when the latest development format is used. These
@@ -40,6 +54,11 @@
 
   (Ian Clatworthy)
 
+* Pyrex version of ``bencode`` support. This provides optimized support
+  for both encoding and decoding, and is now found at ``bzrlib.bencode``,
+  rather than ``bzrlib.utils.bencode``.
+  (Alexander Belchenko, Jelmer Vernooij, John Arbash Meinel)
+
 
 Bug Fixes
 *********
@@ -56,6 +75,12 @@
   the fetched revisions, not in the stacked-on ancestry.
   (John Arbash Meinel)
 
+* Reconcile can now deal with text revisions that originated in revisions 
+  that are ghosts. (Jelmer Vernooij, #336749)
+
+* Support cloning of branches with ghosts in the left hand side history.
+  (Jelmer Vernooij, #248540)
+
 * The ''bzr diff'' now catches OSError from osutils.rmtree and logs a
   helpful message to the trace file, unless the temp directory really was
   removed (which would be very strange).  Since the diff operation has
@@ -85,6 +110,10 @@
 Testing
 *******
 
+* The number of cores is now correctly detected on OSX. (John Szakmeister)
+
+* The number of cores is also detected on Solaris and win32. (Vincent Ladeuil)
+
 
 bzr 1.15
 ########

=== added file 'bzrlib/_bencode_pyx.h'
--- bzrlib/_bencode_pyx.h	1970-01-01 00:00:00 +0000
+++ bzrlib/_bencode_pyx.h	2009-06-04 16:50:33 +0000
@@ -0,0 +1,22 @@
+/* Copyright (C) 2009 Canonical Ltd
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
+/* Simple header providing some macro definitions for _bencode_pyx.pyx
+ */
+
+#define D_UPDATE_TAIL(self, n) (((self)->size -= (n), (self)->tail += (n)))
+#define E_UPDATE_TAIL(self, n) (((self)->size += (n), (self)->tail += (n)))

=== added file 'bzrlib/_bencode_pyx.pyx'
--- bzrlib/_bencode_pyx.pyx	1970-01-01 00:00:00 +0000
+++ bzrlib/_bencode_pyx.pyx	2009-06-05 01:48:32 +0000
@@ -0,0 +1,392 @@
+# Copyright (C) 2007,2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Pyrex implementation for bencode coder/decoder"""
+
+
+cdef extern from "stddef.h":
+    ctypedef unsigned int size_t
+
+cdef extern from "Python.h":
+    ctypedef int  Py_ssize_t
+    int PyInt_CheckExact(object o)
+    int PyLong_CheckExact(object o)
+    int PyString_CheckExact(object o)
+    int PyTuple_CheckExact(object o)
+    int PyList_CheckExact(object o)
+    int PyDict_CheckExact(object o)
+    int PyBool_Check(object o)
+    object PyString_FromStringAndSize(char *v, Py_ssize_t len)
+    char *PyString_AS_STRING(object o) except NULL
+    Py_ssize_t PyString_GET_SIZE(object o) except -1
+    object PyInt_FromString(char *str, char **pend, int base)
+    int Py_GetRecursionLimit()
+    int Py_EnterRecursiveCall(char *)
+    void Py_LeaveRecursiveCall()
+
+    int PyList_Append(object, object) except -1
+
+cdef extern from "stdlib.h":
+    void free(void *memblock)
+    void *malloc(size_t size)
+    void *realloc(void *memblock, size_t size)
+    long strtol(char *, char **, int)
+
+cdef extern from "string.h":
+    void *memcpy(void *dest, void *src, size_t count)
+
+cdef extern from "python-compat.h":
+    int snprintf(char* buffer, size_t nsize, char* fmt, ...)
+
+cdef class Decoder
+cdef class Encoder
+
+cdef extern from "_bencode_pyx.h":
+    void D_UPDATE_TAIL(Decoder, int n)
+    void E_UPDATE_TAIL(Encoder, int n)
+
+
+cdef class Decoder:
+    """Bencode decoder"""
+
+    cdef readonly char *tail
+    cdef readonly int size
+    cdef readonly int _yield_tuples
+    cdef object text
+
+    def __init__(self, s, yield_tuples=0):
+        """Initialize decoder engine.
+        @param  s:  Python string.
+        """
+        if not PyString_CheckExact(s):
+            raise TypeError("String required")
+
+        self.text = s
+        self.tail = PyString_AS_STRING(s)
+        self.size = PyString_GET_SIZE(s)
+        self._yield_tuples = int(yield_tuples)
+
+    def decode(self):
+        result = self._decode_object()
+        if self.size != 0:
+            raise ValueError('junk in stream')
+        return result
+
+    def decode_object(self):
+        return self._decode_object()
+
+    cdef object _decode_object(self):
+        cdef char ch
+
+        if 0 == self.size:
+            raise ValueError('stream underflow')
+
+        if Py_EnterRecursiveCall("_decode_object"):
+            raise RuntimeError("too deeply nested")
+        try:
+            ch = self.tail[0]
+            if c'0' <= ch <= c'9':
+                return self._decode_string()
+            elif ch == c'l':
+                D_UPDATE_TAIL(self, 1)
+                return self._decode_list()
+            elif ch == c'i':
+                D_UPDATE_TAIL(self, 1)
+                return self._decode_int()
+            elif ch == c'd':
+                D_UPDATE_TAIL(self, 1)
+                return self._decode_dict()
+            else:
+                raise ValueError('unknown object type identifier %r' % ch)
+        finally:
+            Py_LeaveRecursiveCall()
+
+    cdef int _read_digits(self, char stop_char) except -1:
+        cdef int i
+        i = 0
+        while ((self.tail[i] >= c'0' and self.tail[i] <= c'9') or
+               self.tail[i] == c'-') and i < self.size:
+            i = i + 1
+
+        if self.tail[i] != stop_char:
+            raise ValueError("Stop character %c not found: %c" % 
+                (stop_char, self.tail[i]))
+        if (self.tail[0] == c'0' or 
+                (self.tail[0] == c'-' and self.tail[1] == c'0')):
+            if i == 1:
+                return i
+            else:
+                raise ValueError # leading zeroes are not allowed
+        return i
+
+    cdef object _decode_int(self):
+        cdef int i
+        i = self._read_digits(c'e')
+        self.tail[i] = 0
+        try:
+            ret = PyInt_FromString(self.tail, NULL, 10)
+        finally:
+            self.tail[i] = c'e'
+        D_UPDATE_TAIL(self, i+1)
+        return ret
+
+    cdef object _decode_string(self):
+        cdef int n
+        cdef char *next_tail
+        # strtol allows leading whitespace, negatives, and leading zeros
+        # however, all callers have already checked that '0' <= tail[0] <= '9'
+        # or they wouldn't have called _decode_string
+        # strtol will stop at trailing whitespace, etc
+        n = strtol(self.tail, &next_tail, 10)
+        if next_tail == NULL or next_tail[0] != c':':
+            raise ValueError('string len not terminated by ":"')
+        # strtol allows leading zeros, so validate that we don't have that
+        if (self.tail[0] == c'0'
+            and (n != 0 or (next_tail - self.tail != 1))):
+            raise ValueError('leading zeros are not allowed')
+        D_UPDATE_TAIL(self, next_tail - self.tail + 1)
+        if n == 0:
+            return ''
+        if n > self.size:
+            raise ValueError('stream underflow')
+        if n < 0:
+            raise ValueError('string size below zero: %d' % n)
+
+        result = PyString_FromStringAndSize(self.tail, n)
+        D_UPDATE_TAIL(self, n)
+        return result
+
+    cdef object _decode_list(self):
+        result = []
+
+        while self.size > 0:
+            if self.tail[0] == c'e':
+                D_UPDATE_TAIL(self, 1)
+                if self._yield_tuples:
+                    return tuple(result)
+                else:
+                    return result
+            else:
+                # As a quick shortcut, check to see if the next object is a
+                # string, since we know that won't be creating recursion
+                # if self.tail[0] >= c'0' and self.tail[0] <= c'9':
+                PyList_Append(result, self._decode_object())
+
+        raise ValueError('malformed list')
+
+    cdef object _decode_dict(self):
+        cdef char ch
+
+        result = {}
+        lastkey = None
+
+        while self.size > 0:
+            ch = self.tail[0]
+            if ch == c'e':
+                D_UPDATE_TAIL(self, 1)
+                return result
+            else:
+                # keys should be strings only
+                if self.tail[0] < c'0' or self.tail[0] > c'9':
+                    raise ValueError('key was not a simple string.')
+                key = self._decode_string()
+                if lastkey >= key:
+                    raise ValueError('dict keys disordered')
+                else:
+                    lastkey = key
+                value = self._decode_object()
+                result[key] = value
+
+        raise ValueError('malformed dict')
+
+
+def bdecode(object s):
+    """Decode string x to Python object"""
+    return Decoder(s).decode()
+
+
+def bdecode_as_tuple(object s):
+    """Decode string x to Python object, using tuples rather than lists."""
+    return Decoder(s, True).decode()
+
+
+class Bencached(object):
+    __slots__ = ['bencoded']
+
+    def __init__(self, s):
+        self.bencoded = s
+
+
+cdef enum:
+    INITSIZE = 1024     # initial size for encoder buffer
+    INT_BUF_SIZE = 32
+
+
+cdef class Encoder:
+    """Bencode encoder"""
+
+    cdef readonly char *tail
+    cdef readonly int size
+    cdef readonly char *buffer
+    cdef readonly int maxsize
+
+    def __init__(self, int maxsize=INITSIZE):
+        """Initialize encoder engine
+        @param  maxsize:    initial size of internal char buffer
+        """
+        cdef char *p
+
+        self.maxsize = 0
+        self.size = 0
+        self.tail = NULL
+
+        p = <char*>malloc(maxsize)
+        if p == NULL:
+            raise MemoryError('Not enough memory to allocate buffer '
+                              'for encoder')
+        self.buffer = p
+        self.maxsize = maxsize
+        self.tail = p
+
+    def __del__(self):
+        free(self.buffer)
+        self.buffer = NULL
+        self.maxsize = 0
+
+    def __str__(self):
+        if self.buffer != NULL and self.size != 0:
+            return PyString_FromStringAndSize(self.buffer, self.size)
+        else:
+            return ''
+
+    cdef int _ensure_buffer(self, int required) except 0:
+        """Ensure that tail of CharTail buffer has enough size.
+        If buffer is not big enough then function try to
+        realloc buffer.
+        """
+        cdef char *new_buffer
+        cdef int   new_size
+
+        if self.size + required < self.maxsize:
+            return 1
+
+        new_size = self.maxsize
+        while new_size < self.size + required:
+            new_size = new_size * 2
+        new_buffer = <char*>realloc(self.buffer, <size_t>new_size)
+        if new_buffer == NULL:
+            raise MemoryError('Cannot realloc buffer for encoder')
+
+        self.buffer = new_buffer
+        self.maxsize = new_size
+        self.tail = &new_buffer[self.size]
+        return 1
+
+    cdef int _encode_int(self, int x) except 0:
+        """Encode int to bencode string iNNNe
+        @param  x:  value to encode
+        """
+        cdef int n
+        self._ensure_buffer(INT_BUF_SIZE)
+        n = snprintf(self.tail, INT_BUF_SIZE, "i%de", x)
+        if n < 0:
+            raise MemoryError('int %d too big to encode' % x)
+        E_UPDATE_TAIL(self, n)
+        return 1
+
+    cdef int _encode_long(self, x) except 0:
+        return self._append_string(''.join(('i', str(x), 'e')))
+
+    cdef int _append_string(self, s) except 0:
+        cdef Py_ssize_t n
+        n = PyString_GET_SIZE(s)
+        self._ensure_buffer(n)
+        memcpy(self.tail, PyString_AS_STRING(s), n)
+        E_UPDATE_TAIL(self, n)
+        return 1
+
+    cdef int _encode_string(self, x) except 0:
+        cdef int n
+        cdef Py_ssize_t x_len
+        x_len = PyString_GET_SIZE(x)
+        self._ensure_buffer(x_len + INT_BUF_SIZE)
+        n = snprintf(self.tail, INT_BUF_SIZE, '%d:', x_len)
+        if n < 0:
+            raise MemoryError('string %s too big to encode' % x)
+        memcpy(<void *>(self.tail+n), PyString_AS_STRING(x), x_len)
+        E_UPDATE_TAIL(self, n + x_len)
+        return 1
+
+    cdef int _encode_list(self, x) except 0:
+        self._ensure_buffer(1)
+        self.tail[0] = c'l'
+        E_UPDATE_TAIL(self, 1)
+
+        for i in x:
+            self.process(i)
+
+        self._ensure_buffer(1)
+        self.tail[0] = c'e'
+        E_UPDATE_TAIL(self, 1)
+        return 1
+
+    cdef int _encode_dict(self, x) except 0:
+        self._ensure_buffer(1)
+        self.tail[0] = c'd'
+        E_UPDATE_TAIL(self, 1)
+
+        keys = x.keys()
+        keys.sort()
+        for k in keys:
+            if not PyString_CheckExact(k):
+                raise TypeError('key in dict should be string')
+            self._encode_string(k)
+            self.process(x[k])
+
+        self._ensure_buffer(1)
+        self.tail[0] = c'e'
+        E_UPDATE_TAIL(self, 1)
+        return 1
+
+    def process(self, object x):
+        if Py_EnterRecursiveCall("encode"):
+            raise RuntimeError("too deeply nested")
+        try:
+            if PyString_CheckExact(x):
+                self._encode_string(x)
+            elif PyInt_CheckExact(x):
+                self._encode_int(x)
+            elif PyLong_CheckExact(x):
+                self._encode_long(x)
+            elif PyList_CheckExact(x) or PyTuple_CheckExact(x):
+                self._encode_list(x)
+            elif PyDict_CheckExact(x):
+                self._encode_dict(x)
+            elif PyBool_Check(x):
+                self._encode_int(int(x))
+            elif isinstance(x, Bencached):
+                self._append_string(x.bencoded)
+            else:
+                raise TypeError('unsupported type %r' % x)
+        finally:
+            Py_LeaveRecursiveCall()
+
+
+def bencode(x):
+    """Encode Python object x to string"""
+    encoder = Encoder()
+    encoder.process(x)
+    return str(encoder)

=== modified file 'bzrlib/_groupcompress_pyx.pyx'
--- bzrlib/_groupcompress_pyx.pyx	2009-04-09 20:23:07 +0000
+++ bzrlib/_groupcompress_pyx.pyx	2009-06-02 21:11:18 +0000
@@ -118,6 +118,9 @@
             self._index = NULL
         safe_free(<void **>&self._source_infos)
 
+    def _has_index(self):
+        return (self._index != NULL)
+
     def add_delta_source(self, delta, unadded_bytes):
         """Add a new delta to the source texts.
 
@@ -171,6 +174,9 @@
         source_location = len(self._sources)
         if source_location >= self._max_num_sources:
             self._expand_sources()
+        if source_location != 0 and self._index == NULL:
+            # We were lazy about populating the index, create it now
+            self._populate_first_index()
         self._sources.append(source)
         c_source = PyString_AS_STRING(source)
         c_source_size = PyString_GET_SIZE(source)
@@ -179,11 +185,24 @@
         src.size = c_source_size
 
         src.agg_offset = self._source_offset + unadded_bytes
-        index = create_delta_index(src, self._index)
         self._source_offset = src.agg_offset + src.size
-        if index != NULL:
-            free_delta_index(self._index)
-            self._index = index
+        # We delay creating the index on the first insert
+        if source_location != 0:
+            index = create_delta_index(src, self._index)
+            if index != NULL:
+                free_delta_index(self._index)
+                self._index = index
+
+    cdef _populate_first_index(self):
+        cdef delta_index *index
+        if len(self._sources) != 1 or self._index != NULL:
+            raise AssertionError('_populate_first_index should only be'
+                ' called when we have a single source and no index yet')
+
+        # We know that self._index is already NULL, so whatever
+        # create_delta_index returns is fine
+        self._index = create_delta_index(&self._source_infos[0], NULL)
+        assert self._index != NULL
 
     cdef _expand_sources(self):
         raise RuntimeError('if we move self._source_infos, then we need to'
@@ -201,7 +220,10 @@
         cdef unsigned long delta_size
 
         if self._index == NULL:
-            return None
+            if len(self._sources) == 0:
+                return None
+            # We were just lazy about generating the index
+            self._populate_first_index()
 
         if not PyString_CheckExact(target_bytes):
             raise TypeError('target is not a str')

=== modified file 'bzrlib/benchmarks/__init__.py'
--- bzrlib/benchmarks/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/benchmarks/__init__.py	2009-05-25 23:39:12 +0000
@@ -190,6 +190,7 @@
                    'bzrlib.benchmarks.bench_rocks',
                    'bzrlib.benchmarks.bench_startup',
                    'bzrlib.benchmarks.bench_status',
+                   'bzrlib.benchmarks.bench_tags',
                    'bzrlib.benchmarks.bench_transform',
                    'bzrlib.benchmarks.bench_workingtree',
                    'bzrlib.benchmarks.bench_sftp',

=== added file 'bzrlib/benchmarks/bench_tags.py'
--- bzrlib/benchmarks/bench_tags.py	1970-01-01 00:00:00 +0000
+++ bzrlib/benchmarks/bench_tags.py	2009-05-27 08:33:37 +0000
@@ -0,0 +1,58 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Test for tags serialization (indirect testing for bencode)"""
+
+
+import time
+
+from bzrlib.benchmarks import Benchmark
+import bzrlib.bencode
+import bzrlib.tag
+
+
+class TagsBencodeBenchmark(Benchmark):
+    """Benchmark for serialization/deserialization of tags"""
+
+    def setUp(self):
+        super(TagsBencodeBenchmark, self).setUp()
+        self.tobj = bzrlib.tag.BasicTags(None)
+        tags = {}
+        revid = 'j.random@example.com-20070812132500-%016d'
+        for i in xrange(100):
+            tags[str(i)] = revid % i
+        self.tags = tags
+        self.bencoded_tags = bzrlib.bencode.bencode(tags)
+
+    def time_N(self, N, kallable, *args, **kwargs):
+        def _func(N, kallable, *args, **kwargs):
+            for i in xrange(N):
+                kallable(*args, **kwargs)
+        self.time(_func, N, kallable, *args, **kwargs)
+
+    def test_serialize_empty_tags(self):
+        # Measure overhead of operation
+        self.time_N(10000, self.tobj._serialize_tag_dict, {})
+
+    def test_deserialize_empty_tags(self):
+        # Measure overhead of operation
+        self.time_N(10000, self.tobj._deserialize_tag_dict, 'de')
+
+    def test_serialize_tags(self):
+        self.time_N(1000, self.tobj._serialize_tag_dict, self.tags)
+
+    def test_deserialize_tags(self):
+        self.time_N(1000, self.tobj._deserialize_tag_dict, self.bencoded_tags)

=== added file 'bzrlib/bencode.py'
--- bzrlib/bencode.py	1970-01-01 00:00:00 +0000
+++ bzrlib/bencode.py	2009-06-03 14:14:31 +0000
@@ -0,0 +1,22 @@
+# Copyright (C) 2007,2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Wrapper around the bencode pyrex and python implementation"""
+
+try:
+    from bzrlib._bencode_pyx import bdecode, bdecode_as_tuple, bencode, Bencached
+except ImportError:
+    from bzrlib.util._bencode_py import bdecode, bdecode_as_tuple, bencode, Bencached

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2009-05-29 10:25:37 +0000
+++ bzrlib/branch.py	2009-06-03 21:31:43 +0000
@@ -1085,18 +1085,14 @@
         source_revno, source_revision_id = self.last_revision_info()
         if revision_id is None:
             revno, revision_id = source_revno, source_revision_id
-        elif source_revision_id == revision_id:
-            # we know the revno without needing to walk all of history
-            revno = source_revno
         else:
-            # To figure out the revno for a random revision, we need to build
-            # the revision history, and count its length.
-            # We don't care about the order, just how long it is.
-            # Alternatively, we could start at the current location, and count
-            # backwards. But there is no guarantee that we will find it since
-            # it may be a merged revision.
-            revno = len(list(self.repository.iter_reverse_revision_history(
-                                                                revision_id)))
+            graph = self.repository.get_graph()
+            try:
+                revno = graph.find_distance_to_null(revision_id, 
+                    [(source_revision_id, source_revno)])
+            except errors.GhostRevisionsHaveNoRevno:
+                # Default to 1, if we can't find anything else
+                revno = 1
         destination.set_last_revision_info(revno, revision_id)
 
     @needs_read_lock
@@ -1147,10 +1143,18 @@
 
         :return: A BranchCheckResult.
         """
+        ret = BranchCheckResult(self)
         mainline_parent_id = None
         last_revno, last_revision_id = self.last_revision_info()
-        real_rev_history = list(self.repository.iter_reverse_revision_history(
-                                last_revision_id))
+        real_rev_history = []
+        try:
+            for revid in self.repository.iter_reverse_revision_history(
+                last_revision_id):
+                real_rev_history.append(revid)
+        except errors.RevisionNotPresent:
+            ret.ghosts_in_mainline = True
+        else:
+            ret.ghosts_in_mainline = False
         real_rev_history.reverse()
         if len(real_rev_history) != last_revno:
             raise errors.BzrCheckError('revno does not match len(mainline)'
@@ -1172,7 +1176,7 @@
                                         "parents of {%s}"
                                         % (mainline_parent_id, revision_id))
             mainline_parent_id = revision_id
-        return BranchCheckResult(self)
+        return ret
 
     def _get_checkout_format(self):
         """Return the most suitable metadir for a checkout of this branch.
@@ -2780,6 +2784,7 @@
 
     def __init__(self, branch):
         self.branch = branch
+        self.ghosts_in_mainline = False
 
     def report_results(self, verbose):
         """Report the check results via trace.note.
@@ -2790,6 +2795,8 @@
         note('checked branch %s format %s',
              self.branch.base,
              self.branch._format)
+        if self.ghosts_in_mainline:
+            note('branch contains ghosts in mainline')
 
 
 class Converter5to6(object):

=== modified file 'bzrlib/bundle/serializer/v4.py'
--- bzrlib/bundle/serializer/v4.py	2009-04-02 15:28:11 +0000
+++ bzrlib/bundle/serializer/v4.py	2009-05-25 23:39:12 +0000
@@ -30,7 +30,7 @@
     serializer,
     )
 from bzrlib.bundle import bundle_data, serializer as bundle_serializer
-from bzrlib.util import bencode
+from bzrlib import bencode
 
 
 class BundleWriter(object):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2009-06-03 04:42:02 +0000
+++ bzrlib/bzrdir.py	2009-06-04 21:25:46 +0000
@@ -2358,26 +2358,95 @@
     def set_branch_format(self, format):
         self._branch_format = format
 
-    def require_stacking(self):
+    def require_stacking(self, stack_on=None, possible_transports=None):
+        """We have a request to stack, try to ensure the formats support it.
+
+        :param stack_on: If supplied, it is the URL to a branch that we want to
+            stack on. Check to see if that format supports stacking before
+            forcing an upgrade.
+        """
+        # Stacking is desired. requested by the target, but does the place it
+        # points at support stacking? If it doesn't then we should
+        # not implicitly upgrade. We check this here.
+        new_repo_format = None
+        new_branch_format = None
+
+        # a bit of state for get_target_branch so that we don't try to open it
+        # 2 times, for both repo *and* branch
+        target = [None, False, None] # target_branch, checked, upgrade anyway
+        def get_target_branch():
+            if target[1]:
+                # We've checked, don't check again
+                return target
+            if stack_on is None:
+                # No target format, that means we want to force upgrading
+                target[:] = [None, True, True]
+                return target
+            try:
+                target_dir = BzrDir.open(stack_on,
+                    possible_transports=possible_transports)
+            except errors.NotBranchError:
+                # Nothing there, don't change formats
+                target[:] = [None, True, False]
+                return target
+            except errors.JailBreak:
+                # JailBreak, JFDI and upgrade anyway
+                target[:] = [None, True, True]
+                return target
+            try:
+                target_branch = target_dir.open_branch()
+            except errors.NotBranchError:
+                # No branch, don't upgrade formats
+                target[:] = [None, True, False]
+                return target
+            target[:] = [target_branch, True, False]
+            return target
+
+        if not (self.repository_format.supports_external_lookups):
+            # We need to upgrade the Repository.
+            target_branch, _, do_upgrade = get_target_branch()
+            if target_branch is None:
+                # We don't have a target branch, should we upgrade anyway?
+                if do_upgrade:
+                    # stack_on is inaccessible, JFDI.
+                    # TODO: bad monkey, hard-coded formats...
+                    if self.repository_format.rich_root_data:
+                        new_repo_format = pack_repo.RepositoryFormatKnitPack5RichRoot()
+                    else:
+                        new_repo_format = pack_repo.RepositoryFormatKnitPack5()
+            else:
+                # If the target already supports stacking, then we know the
+                # project is already able to use stacking, so auto-upgrade
+                # for them
+                new_repo_format = target_branch.repository._format
+                if not new_repo_format.supports_external_lookups:
+                    # target doesn't, source doesn't, so don't auto upgrade
+                    # repo
+                    new_repo_format = None
+            if new_repo_format is not None:
+                self.repository_format = new_repo_format
+                note('Source repository format does not support stacking,'
+                     ' using format:\n  %s',
+                     new_repo_format.get_format_description())
+
         if not self.get_branch_format().supports_stacking():
-            # We need to make a stacked branch, but the default format for the
-            # target doesn't support stacking.  So force a branch that *can*
-            # support stacking.
-            from bzrlib.branch import BzrBranchFormat7
-            branch_format = BzrBranchFormat7()
-            self.set_branch_format(branch_format)
-            mutter("using %r for stacking" % (branch_format,))
-            from bzrlib.repofmt import pack_repo
-            if self.repository_format.rich_root_data:
-                bzrdir_format_name = '1.6.1-rich-root'
-                repo_format = pack_repo.RepositoryFormatKnitPack5RichRoot()
+            # We just checked the repo, now lets check if we need to
+            # upgrade the branch format
+            target_branch, _, do_upgrade = get_target_branch()
+            if target_branch is None:
+                if do_upgrade:
+                    # TODO: bad monkey, hard-coded formats...
+                    new_branch_format = branch.BzrBranchFormat7()
             else:
-                bzrdir_format_name = '1.6'
-                repo_format = pack_repo.RepositoryFormatKnitPack5()
-            note('Source format does not support stacking, using format:'
-                 ' \'%s\'\n  %s\n',
-                 bzrdir_format_name, repo_format.get_format_description())
-            self.repository_format = repo_format
+                new_branch_format = target_branch._format
+                if not new_branch_format.supports_stacking():
+                    new_branch_format = None
+            if new_branch_format is not None:
+                # Does support stacking, use its format.
+                self.set_branch_format(new_branch_format)
+                note('Source branch format does not support stacking,'
+                     ' using format:\n  %s',
+                     new_branch_format.get_format_description())
 
     def get_converter(self, format=None):
         """See BzrDirFormat.get_converter()."""
@@ -3532,54 +3601,9 @@
         """
         stack_on = self._get_full_stack_on()
         if stack_on:
-            # Stacking is desired. requested by the target, but does the place it
-            # points at support stacking? If it doesn't then we should
-            # not implicitly upgrade. We check this here.
             format = self._bzrdir._format
-            if not (format.repository_format.supports_external_lookups
-                and format.get_branch_format().supports_stacking()):
-                # May need to upgrade - but only do if the target also
-                # supports stacking. Note that this currently wastes
-                # network round trips to check - but we only do this
-                # when the source can't stack so it will fade away
-                # as people do upgrade.
-                branch_format = None
-                repo_format = None
-                try:
-                    target_dir = BzrDir.open(stack_on,
-                        possible_transports=[self._bzrdir.root_transport])
-                except errors.NotBranchError:
-                    # Nothing there, don't change formats
-                    pass
-                except errors.JailBreak:
-                    # stack_on is inaccessible, JFDI.
-                    if format.repository_format.rich_root_data:
-                        repo_format = pack_repo.RepositoryFormatKnitPack6RichRoot()
-                    else:
-                        repo_format = pack_repo.RepositoryFormatKnitPack6()
-                    branch_format = branch.BzrBranchFormat7()
-                else:
-                    try:
-                        target_branch = target_dir.open_branch()
-                    except errors.NotBranchError:
-                        # No branch, don't change formats
-                        pass
-                    else:
-                        branch_format = target_branch._format
-                        repo_format = target_branch.repository._format
-                        if not (branch_format.supports_stacking()
-                            and repo_format.supports_external_lookups):
-                            # Doesn't stack itself, don't force an upgrade
-                            branch_format = None
-                            repo_format = None
-                if branch_format and repo_format:
-                    # Does support stacking, use its format.
-                    format.repository_format = repo_format
-                    format.set_branch_format(branch_format)
-                    note('Source format does not support stacking, '
-                        'using format: \'%s\'\n  %s\n',
-                        branch_format.get_format_description(),
-                        repo_format.get_format_description())
+            format.require_stacking(stack_on=stack_on,
+                                    possible_transports=[self._bzrdir.root_transport])
             if not self._require_stacking:
                 # We have picked up automatic stacking somewhere.
                 note('Using default stacking branch %s at %s', self._stack_on,
@@ -3792,6 +3816,18 @@
     experimental=True,
     )
 
+format_registry.register_metadir('development7-rich-root',
+    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK2',
+    help='pack-1.9 with 255-way hashed CHK inv, bencode revision, group compress, '
+        'rich roots. Please read '
+        'http://doc.bazaar-vcs.org/latest/developers/development-repo.html '
+        'before use.',
+    branch_format='bzrlib.branch.BzrBranchFormat7',
+    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
+    hidden=True,
+    experimental=True,
+    )
+
 # The following format should be an alias for the rich root equivalent 
 # of the default format
 format_registry.register_metadir('default-rich-root',

=== modified file 'bzrlib/cache_utf8.py'
--- bzrlib/cache_utf8.py	2009-03-23 14:59:43 +0000
+++ bzrlib/cache_utf8.py	2009-06-03 19:51:50 +0000
@@ -22,8 +22,8 @@
 import codecs
 
 
-_utf8_encode = codecs.getencoder("utf-8")
-_utf8_decode = codecs.getdecoder("utf-8")
+_utf8_encode = codecs.utf_8_encode
+_utf8_decode = codecs.utf_8_decode
 def _utf8_decode_with_None(bytestring, _utf8_decode=_utf8_decode):
     """wrap _utf8_decode to support None->None for optional strings.
 

=== modified file 'bzrlib/chk_map.py'
--- bzrlib/chk_map.py	2009-05-13 21:59:57 +0000
+++ bzrlib/chk_map.py	2009-06-05 18:03:40 +0000
@@ -955,34 +955,99 @@
         # prefix is the key in self._items to use, key_filter is the key_filter
         # entries that would match this node
         keys = {}
+        shortcut = False
         if key_filter is None:
+            # yielding all nodes, yield whatever we have, and queue up a read
+            # for whatever we are missing
+            shortcut = True
             for prefix, node in self._items.iteritems():
-                if type(node) == tuple:
+                if type(node) is tuple:
                     keys[node] = (prefix, None)
                 else:
                     yield node, None
-        else:
-            # XXX defaultdict ?
+        elif len(key_filter) == 1:
+            # Technically, this path could also be handled by the first check
+            # in 'self._node_width' in length_filters. However, we can handle
+            # this case without spending any time building up the
+            # prefix_to_keys, etc state.
+
+            # This is a bit ugly, but TIMEIT showed it to be by far the fastest
+            # 0.626us   list(key_filter)[0]
+            #       is a func() for list(), 2 mallocs, and a getitem
+            # 0.489us   [k for k in key_filter][0]
+            #       still has the mallocs, avoids the func() call
+            # 0.350us   iter(key_filter).next()
+            #       has a func() call, and mallocs an iterator
+            # 0.125us   for key in key_filter: pass
+            #       no func() overhead, might malloc an iterator
+            # 0.105us   for key in key_filter: break
+            #       no func() overhead, might malloc an iterator, probably
+            #       avoids checking an 'else' clause as part of the for
+            for key in key_filter:
+                break
+            search_prefix = self._search_prefix_filter(key)
+            if len(search_prefix) == self._node_width:
+                # This item will match exactly, so just do a dict lookup, and
+                # see what we can return
+                shortcut = True
+                try:
+                    node = self._items[search_prefix]
+                except KeyError:
+                    # A given key can only match 1 child node, if it isn't
+                    # there, then we can just return nothing
+                    return
+                if node.__class__ is tuple:
+                    keys[node] = (search_prefix, [key])
+                else:
+                    # This is loaded, and the only thing that can match,
+                    # return
+                    yield node, [key]
+                    return
+        if not shortcut:
+            # First, convert all keys into a list of search prefixes
+            # Aggregate common prefixes, and track the keys they come from
             prefix_to_keys = {}
             length_filters = {}
             for key in key_filter:
-                search_key = self._search_prefix_filter(key)
+                search_prefix = self._search_prefix_filter(key)
                 length_filter = length_filters.setdefault(
-                                    len(search_key), set())
-                length_filter.add(search_key)
-                prefix_to_keys.setdefault(search_key, []).append(key)
-            length_filters = length_filters.items()
-            for prefix, node in self._items.iteritems():
-                node_key_filter = []
-                for length, length_filter in length_filters:
-                    sub_prefix = prefix[:length]
-                    if sub_prefix in length_filter:
-                        node_key_filter.extend(prefix_to_keys[sub_prefix])
-                if node_key_filter: # this key matched something, yield it
+                                    len(search_prefix), set())
+                length_filter.add(search_prefix)
+                prefix_to_keys.setdefault(search_prefix, []).append(key)
+
+            if (self._node_width in length_filters
+                and len(length_filters) == 1):
+                # all of the search prefixes match exactly _node_width. This
+                # means that everything is an exact match, and we can do a
+                # lookup into self._items, rather than iterating over the items
+                # dict.
+                search_prefixes = length_filters[self._node_width]
+                for search_prefix in search_prefixes:
+                    try:
+                        node = self._items[search_prefix]
+                    except KeyError:
+                        # We can ignore this one
+                        continue
+                    node_key_filter = prefix_to_keys[search_prefix]
                     if type(node) == tuple:
-                        keys[node] = (prefix, node_key_filter)
+                        keys[node] = (search_prefix, node_key_filter)
                     else:
                         yield node, node_key_filter
+            else:
+                # The slow way. We walk every item in self._items, and check to
+                # see if there are any matches
+                length_filters = length_filters.items()
+                for prefix, node in self._items.iteritems():
+                    node_key_filter = []
+                    for length, length_filter in length_filters:
+                        sub_prefix = prefix[:length]
+                        if sub_prefix in length_filter:
+                            node_key_filter.extend(prefix_to_keys[sub_prefix])
+                    if node_key_filter: # this key matched something, yield it
+                        if type(node) == tuple:
+                            keys[node] = (prefix, node_key_filter)
+                        else:
+                            yield node, node_key_filter
         if keys:
             # Look in the page cache for some more bytes
             found_keys = set()

=== modified file 'bzrlib/chk_serializer.py'
--- bzrlib/chk_serializer.py	2009-04-09 20:23:07 +0000
+++ bzrlib/chk_serializer.py	2009-06-04 21:50:04 +0000
@@ -16,14 +16,125 @@
 
 """Serializer object for CHK based inventory storage."""
 
+from cStringIO import (
+    StringIO,
+    )
+
 from bzrlib import (
+    bencode,
+    cache_utf8,
     inventory,
+    osutils,
+    revision as _mod_revision,
     xml5,
     xml6,
     )
 
 
-class CHKSerializerSubtree(xml6.Serializer_v6):
+def _validate_properties(props, _decode=cache_utf8._utf8_decode):
+    # TODO: we really want an 'isascii' check for key
+    # Cast the utf8 properties into Unicode 'in place'
+    for key, value in props.iteritems():
+        props[key] = _decode(value)[0]
+    return props
+
+
+def _is_format_10(value):
+    if value != 10:
+        raise ValueError('Format number was not recognized, expected 10 got %d'
+                         % (value,))
+    return 10
+
+
+class BEncodeRevisionSerializer1(object):
+    """Simple revision serializer based around bencode.
+    """
+
+    # Maps {key:(Revision attribute, bencode_type, validator)}
+    # This tells us what kind we expect bdecode to create, what variable on
+    # Revision we should be using, and a function to call to validate/transform
+    # the type.
+    # TODO: add a 'validate_utf8' for things like revision_id and file_id
+    #       and a validator for parent-ids
+    _schema = {'format': (None, int, _is_format_10),
+               'committer': ('committer', str, cache_utf8.decode),
+               'timezone': ('timezone', int, None),
+               'timestamp': ('timestamp', str, float),
+               'revision-id': ('revision_id', str, None),
+               'parent-ids': ('parent_ids', list, None),
+               'inventory-sha1': ('inventory_sha1', str, None),
+               'message': ('message', str, cache_utf8.decode),
+               'properties': ('properties', dict, _validate_properties),
+    }
+
+    def write_revision_to_string(self, rev):
+        encode_utf8 = cache_utf8._utf8_encode
+        # Use a list of tuples rather than a dict
+        # This lets us control the ordering, so that we are able to create
+        # smaller deltas
+        ret = [
+            ("format", 10),
+            ("committer", encode_utf8(rev.committer)[0]),
+        ]
+        if rev.timezone is not None:
+            ret.append(("timezone", rev.timezone))
+        # For bzr revisions, the most common property is just 'branch-nick'
+        # which changes infrequently.
+        revprops = {}
+        for key, value in rev.properties.iteritems():
+            revprops[key] = encode_utf8(value)[0]
+        ret.append(('properties', revprops))
+        ret.extend([
+            ("timestamp", "%.3f" % rev.timestamp),
+            ("revision-id", rev.revision_id),
+            ("parent-ids", rev.parent_ids),
+            ("inventory-sha1", rev.inventory_sha1),
+            ("message", encode_utf8(rev.message)[0]),
+        ])
+        return bencode.bencode(ret)
+
+    def write_revision(self, rev, f):
+        f.write(self.write_revision_to_string(rev))
+
+    def read_revision_from_string(self, text):
+        # TODO: consider writing a Revision decoder, rather than using the
+        #       generic bencode decoder
+        #       However, to decode all 25k revisions of bzr takes approx 1.3s
+        #       If we remove all extra validation that goes down to about 1.2s.
+        #       Of that time, probably 0.6s is spend in bencode.bdecode().
+        #       Regardless 'time bzr log' of everything is 7+s, so 1.3s to
+        #       extract revision texts isn't a majority of time.
+        ret = bencode.bdecode(text)
+        if not isinstance(ret, list):
+            raise ValueError("invalid revision text")
+        schema = self._schema
+        # timezone is allowed to be missing, but should be set
+        bits = {'timezone': None}
+        for key, value in ret:
+            # Will raise KeyError if not a valid part of the schema, or an
+            # entry is given 2 times.
+            var_name, expected_type, validator = schema[key]
+            if value.__class__ is not expected_type:
+                raise ValueError('key %s did not conform to the expected type'
+                                 ' %s, but was %s'
+                                 % (key, expected_type, type(value)))
+            if validator is not None:
+                value = validator(value)
+            bits[var_name] = value
+        if len(bits) != len(schema):
+            missing = [key for key, (var_name, _, _) in schema.iteritems()
+                       if var_name not in bits]
+            raise ValueError('Revision text was missing expected keys %s.'
+                             ' text %r' % (missing, text))
+        del bits[None]  # Get rid of 'format' since it doesn't get mapped
+        rev = _mod_revision.Revision(**bits)
+        return rev
+
+    def read_revision(self, f):
+        return self.read_revision_from_string(f.read())
+
+
+class CHKSerializerSubtree(BEncodeRevisionSerializer1, xml6.Serializer_v6):
     """A CHKInventory based serializer that supports tree references"""
 
     supported_kinds = set(['file', 'directory', 'symlink', 'tree-reference'])
@@ -64,3 +175,12 @@
 
 
 chk_serializer_255_bigpage = CHKSerializer(65536, 'hash-255-way')
+
+
+class CHKBEncodeSerializer(BEncodeRevisionSerializer1, CHKSerializer):
+    """A CHKInventory and BEncode based serializer with 'plain' behaviour."""
+
+    format_num = '10'
+
+
+chk_bencode_serializer = CHKBEncodeSerializer(65536, 'hash-255-way')

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2009-05-29 10:25:37 +0000
+++ bzrlib/groupcompress.py	2009-06-05 04:50:30 +0000
@@ -324,7 +324,11 @@
                 raise ValueError('invalid content_len %d for record @ pos %d'
                                  % (content_len, pos - len_len - 1))
             if kind == 'f': # Fulltext
-                result.append(('f', content_len))
+                if include_text:
+                    text = self._content[pos:pos+content_len]
+                    result.append(('f', content_len, text))
+                else:
+                    result.append(('f', content_len))
             elif kind == 'd': # Delta
                 delta_content = self._content[pos:pos+content_len]
                 delta_info = []
@@ -339,7 +343,11 @@
                         (offset, length,
                          delta_pos) = decode_copy_instruction(delta_content, c,
                                                               delta_pos)
-                        delta_info.append(('c', offset, length))
+                        if include_text:
+                            text = self._content[offset:offset+length]
+                            delta_info.append(('c', offset, length, text))
+                        else:
+                            delta_info.append(('c', offset, length))
                         measured_len += length
                     else: # Insert
                         if include_text:
@@ -746,6 +754,14 @@
 
         After calling this, the compressor should no longer be used
         """
+        # TODO: this causes us to 'bloat' to 2x the size of content in the
+        #       group. This has an impact for 'commit' of large objects.
+        #       One possibility is to use self._content_chunks, and be lazy and
+        #       only fill out self._content as a full string when we actually
+        #       need it. That would at least drop the peak memory consumption
+        #       for 'commit' down to ~1x the size of the largest file, at a
+        #       cost of increased complexity within this code. 2x is still <<
+        #       3x the size of the largest file, so we are doing ok.
         content = ''.join(self.chunks)
         self.chunks = None
         self._delta_index = None

=== modified file 'bzrlib/mail_client.py'
--- bzrlib/mail_client.py	2009-06-03 19:05:40 +0000
+++ bzrlib/mail_client.py	2009-06-04 21:05:44 +0000
@@ -155,7 +155,7 @@
                       extension, **kwargs)
 
     def _compose(self, prompt, to, subject, attach_path, mime_subtype,
-                extension, body=None):
+                 extension, body=None, from_=None):
         """Invoke a mail client as a commandline process.
 
         Overridden by MAPIClient.
@@ -166,6 +166,8 @@
             "text", but the precise subtype can be specified here
         :param extension: A file extension (including period) associated with
             the attachment type.
+        :param body: Optional body text.
+        :param from_: Optional From: header.
         """
         for name in self._get_client_commands():
             cmdline = [self._encode_path(name, 'executable')]
@@ -173,6 +175,8 @@
                 kwargs = {'body': body}
             else:
                 kwargs = {}
+            if from_ is not None:
+                kwargs['from_'] = from_
             cmdline.extend(self._get_compose_commandline(to, subject,
                                                          attach_path,
                                                          **kwargs))
@@ -331,25 +335,45 @@
 class Claws(ExternalMailClient):
     """Claws mail client."""
 
+    supports_body = True
+
     _client_commands = ['claws-mail']
 
-    def _get_compose_commandline(self, to, subject, attach_path):
+    def _get_compose_commandline(self, to, subject, attach_path, body=None,
+                                 from_=None):
         """See ExternalMailClient._get_compose_commandline"""
-        compose_url = ['mailto:']
-        if to is not None:
-            compose_url.append(self._encode_safe(to))
-        compose_url.append('?')
+        compose_url = []
+        if from_ is not None:
+            compose_url.append('from=' + urllib.quote(from_))
         if subject is not None:
             # Don't use urllib.quote_plus because Claws doesn't seem
             # to recognise spaces encoded as "+".
             compose_url.append(
-                'subject=%s' % urllib.quote(self._encode_safe(subject)))
+                'subject=' + urllib.quote(self._encode_safe(subject)))
+        if body is not None:
+            compose_url.append(
+                'body=' + urllib.quote(self._encode_safe(body)))
+        # to must be supplied for the claws-mail --compose syntax to work.
+        if to is None:
+            raise errors.NoMailAddressSpecified()
+        compose_url = 'mailto:%s?%s' % (
+            self._encode_safe(to), '&'.join(compose_url))
         # Collect command-line options.
-        message_options = ['--compose', ''.join(compose_url)]
+        message_options = ['--compose', compose_url]
         if attach_path is not None:
             message_options.extend(
                 ['--attach', self._encode_path(attach_path, 'attachment')])
         return message_options
+
+    def _compose(self, prompt, to, subject, attach_path, mime_subtype,
+                 extension, body=None, from_=None):
+        """See ExternalMailClient._compose"""
+        if from_ is None:
+            from_ = self.config.get_user_option('email')
+        super(Claws, self)._compose(prompt, to, subject, attach_path,
+                                    mime_subtype, extension, body, from_)
+
+
 mail_client_registry.register('claws', Claws,
                               help=Claws.__doc__)
 

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2009-04-04 02:50:01 +0000
+++ bzrlib/multiparent.py	2009-05-25 23:39:12 +0000
@@ -28,7 +28,7 @@
     trace,
     ui,
     )
-from bzrlib.util import bencode
+from bzrlib import bencode
 """)
 from bzrlib.tuned_gzip import GzipFile
 

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2009-05-23 04:55:52 +0000
+++ bzrlib/osutils.py	2009-06-05 07:10:13 +0000
@@ -38,6 +38,7 @@
 from shutil import (
     rmtree,
     )
+import subprocess
 import tempfile
 from tempfile import (
     mkdtemp,
@@ -1826,3 +1827,54 @@
         finally:
             termios.tcsetattr(fd, termios.TCSADRAIN, settings)
         return ch
+
+
+if sys.platform == 'linux2':
+    def _local_concurrency():
+        concurrency = None
+        prefix = 'processor'
+        for line in file('/proc/cpuinfo', 'rb'):
+            if line.startswith(prefix):
+                concurrency = int(line[line.find(':')+1:]) + 1
+        return concurrency
+elif sys.platform == 'darwin':
+    def _local_concurrency():
+        return subprocess.Popen(['sysctl', '-n', 'hw.availcpu'],
+                                stdout=subprocess.PIPE).communicate()[0]
+elif sys.platform == 'sunos5':
+    def _local_concurrency():
+        return subprocess.Popen(['psrinfo', '-p',],
+                                stdout=subprocess.PIPE).communicate()[0]
+elif sys.platform == "win32":
+    def _local_concurrency():
+        # This appears to return the number of cores.
+        return os.environ.get('NUMBER_OF_PROCESSORS')
+else:
+    def _local_concurrency():
+        # Who knows ?
+        return None
+
+
+_cached_local_concurrency = None
+
+def local_concurrency(use_cache=True):
+    """Return how many processes can be run concurrently.
+
+    Rely on platform specific implementations and default to 1 (one) if
+    anything goes wrong.
+    """
+    global _cached_local_concurrency
+    if _cached_local_concurrency is not None and use_cache:
+        return _cached_local_concurrency
+
+    try:
+        concurrency = _local_concurrency()
+    except (OSError, IOError):
+        concurrency = None
+    try:
+        concurrency = int(concurrency)
+    except (TypeError, ValueError):
+        concurrency = 1
+    if use_cache:
+        _cached_concurrency = concurrency
+    return concurrency

=== modified file 'bzrlib/python-compat.h'
--- bzrlib/python-compat.h	2009-03-23 14:59:43 +0000
+++ bzrlib/python-compat.h	2009-05-26 15:46:31 +0000
@@ -67,5 +67,10 @@
     #include "arpa/inet.h"
 #endif
 
+#include <stdio.h>
+
+#ifdef _MSC_VER
+#define  snprintf  _snprintf
+#endif
 
 #endif /* _BZR_PYTHON_COMPAT_H */

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2009-04-09 20:23:07 +0000
+++ bzrlib/reconcile.py	2009-06-02 01:27:37 +0000
@@ -137,8 +137,13 @@
     def _reconcile_revision_history(self):
         repo = self.branch.repository
         last_revno, last_revision_id = self.branch.last_revision_info()
-        real_history = list(repo.iter_reverse_revision_history(
-                                last_revision_id))
+        real_history = []
+        try:
+            for revid in repo.iter_reverse_revision_history(
+                    last_revision_id):
+                real_history.append(revid)
+        except errors.RevisionNotPresent:
+            pass # Hit a ghost left hand parent
         real_history.reverse()
         if last_revno != len(real_history):
             self.fixed_history = True

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2009-05-28 09:37:15 +0000
+++ bzrlib/remote.py	2009-06-03 01:37:27 +0000
@@ -20,6 +20,7 @@
 import bz2
 
 from bzrlib import (
+    bencode,
     branch,
     bzrdir,
     config,
@@ -45,7 +46,6 @@
 from bzrlib.smart import client, vfs, repository as smart_repo
 from bzrlib.revision import ensure_null, NULL_REVISION
 from bzrlib.trace import mutter, note, warning
-from bzrlib.util import bencode
 
 
 class _RpcHelper(object):

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2009-06-01 18:13:46 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2009-06-03 01:43:22 +0000
@@ -1049,3 +1049,23 @@
                 'Does not support nested trees', target_format)
 
 
+
+class RepositoryFormatCHK2(RepositoryFormatCHK1):
+    """A CHK repository that uses the bencode revision serializer."""
+
+    _serializer = chk_serializer.chk_bencode_serializer
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir('development7-rich-root')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return ('Bazaar development format - chk repository with bencode '
+                'revision serialization (needs bzr.dev from 1.15)\n')
+
+

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2009-05-28 09:35:53 +0000
+++ bzrlib/repofmt/pack_repo.py	2009-06-05 03:56:05 +0000
@@ -1306,7 +1306,7 @@
         # space (we only topo sort the revisions, which is smaller).
         topo_order = tsort.topo_sort(ancestors)
         rev_order = dict(zip(topo_order, range(len(topo_order))))
-        bad_texts.sort(key=lambda key:rev_order[key[0][1]])
+        bad_texts.sort(key=lambda key:rev_order.get(key[0][1], 0))
         transaction = repo.get_transaction()
         file_id_index = GraphIndexPrefixAdapter(
             self.new_pack.text_index,
@@ -2289,7 +2289,11 @@
 
     def _resume_write_group(self, tokens):
         self._start_write_group()
-        self._pack_collection._resume_write_group(tokens)
+        try:
+            self._pack_collection._resume_write_group(tokens)
+        except errors.UnresumableWriteGroup:
+            self._abort_write_group()
+            raise
         for pack in self._pack_collection._resumed_packs:
             self.revisions._index.scan_unvalidated_index(pack.revision_index)
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-06-01 18:13:46 +0000
+++ bzrlib/repository.py	2009-06-04 21:25:46 +0000
@@ -2245,13 +2245,11 @@
         while True:
             if next_id in (None, _mod_revision.NULL_REVISION):
                 return
+            try:
+                parents = graph.get_parent_map([next_id])[next_id]
+            except KeyError:
+                raise errors.RevisionNotPresent(next_id, self)
             yield next_id
-            # Note: The following line may raise KeyError in the event of
-            # truncated history. We decided not to have a try:except:raise
-            # RevisionNotPresent here until we see a use for it, because of the
-            # cost in an inner loop that is by its very nature O(history).
-            # Robert Collins 20080326
-            parents = graph.get_parent_map([next_id])[next_id]
             if len(parents) == 0:
                 return
             else:
@@ -3062,6 +3060,13 @@
     'RepositoryFormatCHK1',
     )
 
+format_registry.register_lazy(
+    'Bazaar development format - chk repository with bencode revision '
+        'serialization (needs bzr.dev from 1.15)\n',
+    'bzrlib.repofmt.groupcompress_repo',
+    'RepositoryFormatCHK2',
+    )
+
 
 class InterRepository(InterObject):
     """This class represents operations taking place between two repositories.

=== modified file 'bzrlib/serializer.py'
--- bzrlib/serializer.py	2009-04-03 21:50:40 +0000
+++ bzrlib/serializer.py	2009-06-04 21:26:47 +0000
@@ -75,3 +75,7 @@
 format_registry.register_lazy('6', 'bzrlib.xml6', 'serializer_v6')
 format_registry.register_lazy('7', 'bzrlib.xml7', 'serializer_v7')
 format_registry.register_lazy('8', 'bzrlib.xml8', 'serializer_v8')
+format_registry.register_lazy('9', 'bzrlib.chk_serializer',
+    'chk_serializer_255_bigpage')
+format_registry.register_lazy('10', 'bzrlib.chk_serializer',
+    'chk_bencode_serializer')

=== modified file 'bzrlib/shelf.py'
--- bzrlib/shelf.py	2009-03-23 14:59:43 +0000
+++ bzrlib/shelf.py	2009-05-26 09:20:17 +0000
@@ -19,6 +19,7 @@
 import re
 
 from bzrlib import (
+    bencode,
     errors,
     merge,
     merge3,
@@ -28,7 +29,6 @@
     ui,
     workingtree,
 )
-from bzrlib.util import bencode
 
 
 class ShelfCreator(object):

=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2009-03-23 14:59:43 +0000
+++ bzrlib/smart/protocol.py	2009-05-26 09:20:17 +0000
@@ -29,7 +29,7 @@
 from bzrlib import errors
 from bzrlib.smart import message, request
 from bzrlib.trace import log_exception_quietly, mutter
-from bzrlib.util.bencode import bdecode_as_tuple, bencode
+from bzrlib.bencode import bdecode_as_tuple, bencode
 
 
 # Protocol version strings.  These are sent as prefixes of bzr requests and

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2009-06-01 18:13:46 +0000
+++ bzrlib/smart/repository.py	2009-06-03 01:37:27 +0000
@@ -26,6 +26,7 @@
 import threading
 
 from bzrlib import (
+    bencode,
     errors,
     graph,
     osutils,
@@ -39,7 +40,6 @@
     )
 from bzrlib.repository import _strip_NULL_ghosts, network_format_registry
 from bzrlib import revision as _mod_revision
-from bzrlib.util import bencode
 from bzrlib.versionedfile import NetworkRecordStream, record_to_fulltext_bytes
 
 

=== modified file 'bzrlib/tag.py'
--- bzrlib/tag.py	2009-05-06 05:36:28 +0000
+++ bzrlib/tag.py	2009-05-26 09:20:17 +0000
@@ -28,12 +28,8 @@
 
 from warnings import warn
 
-from bzrlib.lazy_import import lazy_import
-lazy_import(globals(), """
-from bzrlib.util import bencode
-""")
-
 from bzrlib import (
+    bencode,
     errors,
     trace,
     )

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-05-23 04:55:52 +0000
+++ bzrlib/tests/__init__.py	2009-06-05 07:14:08 +0000
@@ -2753,7 +2753,7 @@
 
 
 def fork_decorator(suite):
-    concurrency = local_concurrency()
+    concurrency = osutils.local_concurrency()
     if concurrency == 1:
         return suite
     from testtools import ConcurrentTestSuite
@@ -2762,7 +2762,7 @@
 
 
 def subprocess_decorator(suite):
-    concurrency = local_concurrency()
+    concurrency = osutils.local_concurrency()
     if concurrency == 1:
         return suite
     from testtools import ConcurrentTestSuite
@@ -2954,7 +2954,7 @@
     :return: An iterable of TestCase-like objects which can each have
         run(result) called on them to feed tests to result.
     """
-    concurrency = local_concurrency()
+    concurrency = osutils.local_concurrency()
     result = []
     from subunit import TestProtocolClient, ProtocolTestCase
     class TestInOtherProcess(ProtocolTestCase):
@@ -3003,7 +3003,7 @@
     :return: An iterable of TestCase-like objects which can each have
         run(result) called on them to feed tests to result.
     """
-    concurrency = local_concurrency()
+    concurrency = osutils.local_concurrency()
     result = []
     from subunit import TestProtocolClient, ProtocolTestCase
     class TestInSubprocess(ProtocolTestCase):
@@ -3049,24 +3049,6 @@
     return result
 
 
-def cpucount(content):
-    lines = content.splitlines()
-    prefix = 'processor'
-    for line in lines:
-        if line.startswith(prefix):
-            concurrency = int(line[line.find(':')+1:]) + 1
-    return concurrency
-
-
-def local_concurrency():
-    try:
-        content = file('/proc/cpuinfo', 'rb').read()
-        concurrency = cpucount(content)
-    except Exception, e:
-        concurrency = 1
-    return concurrency
-
-
 class BZRTransformingResult(unittest.TestResult):
 
     def __init__(self, target):
@@ -3370,6 +3352,7 @@
                    'bzrlib.tests.test_api',
                    'bzrlib.tests.test_atomicfile',
                    'bzrlib.tests.test_bad_files',
+                   'bzrlib.tests.test_bencode',
                    'bzrlib.tests.test_bisect_multi',
                    'bzrlib.tests.test_branch',
                    'bzrlib.tests.test_branchbuilder',
@@ -3380,6 +3363,7 @@
                    'bzrlib.tests.test__chunks_to_lines',
                    'bzrlib.tests.test_cache_utf8',
                    'bzrlib.tests.test_chk_map',
+                   'bzrlib.tests.test_chk_serializer',
                    'bzrlib.tests.test_chunk_writer',
                    'bzrlib.tests.test_clean_tree',
                    'bzrlib.tests.test_commands',
@@ -3517,7 +3501,6 @@
                    'bzrlib.tests.test_xml',
                    'bzrlib.tests.tree_implementations',
                    'bzrlib.tests.workingtree_implementations',
-                   'bzrlib.util.tests.test_bencode',
                    ]
 
     loader = TestUtil.TestLoader()

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2009-04-15 04:45:06 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2009-06-03 20:03:55 +0000
@@ -237,9 +237,10 @@
             ['branch', '--stacked', 'trunk', 'shallow'])
         # We should notify the user that we upgraded their format
         self.assertEqualDiff(
-            'Source format does not support stacking, using format: \'1.6\'\n'
+            'Source repository format does not support stacking, using format:\n'
             '  Packs 5 (adds stacking support, requires bzr 1.6)\n'
-            '\n'
+            'Source branch format does not support stacking, using format:\n'
+            '  Branch format 7\n'
             'Created new stacked branch referring to %s.\n' % (trunk.base,),
             err)
 
@@ -249,10 +250,10 @@
             ['branch', '--stacked', 'trunk', 'shallow'])
         # We should notify the user that we upgraded their format
         self.assertEqualDiff(
-            'Source format does not support stacking, using format:'
-            ' \'1.6.1-rich-root\'\n'
+            'Source repository format does not support stacking, using format:\n'
             '  Packs 5 rich-root (adds stacking support, requires bzr 1.6.1)\n'
-            '\n'
+            'Source branch format does not support stacking, using format:\n'
+            '  Branch format 7\n'
             'Created new stacked branch referring to %s.\n' % (trunk.base,),
             err)
 

=== modified file 'bzrlib/tests/branch_implementations/test_check.py'
--- bzrlib/tests/branch_implementations/test_check.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/branch_implementations/test_check.py	2009-05-28 15:22:13 +0000
@@ -67,4 +67,11 @@
         result.report_results(verbose=True)
         result.report_results(verbose=False)
 
+    def test_check_detects_ghosts_in_mainline(self):
+        tree = self.make_branch_and_tree('test')
+        tree.set_parent_ids(['thisisaghost'], allow_leftmost_as_ghost=True)
+        r1 = tree.commit('one')
+        r2 = tree.commit('two')
+        result = tree.branch.check()
+        self.assertEquals(True, result.ghosts_in_mainline)
 

=== modified file 'bzrlib/tests/branch_implementations/test_reconcile.py'
--- bzrlib/tests/branch_implementations/test_reconcile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/branch_implementations/test_reconcile.py	2009-06-02 01:27:37 +0000
@@ -70,3 +70,13 @@
         a_branch = self.make_branch('a_branch')
         a_branch.reconcile(thorough=False)
         a_branch.reconcile(thorough=True)
+
+    def test_reconcile_handles_ghosts_in_revhistory(self):
+        tree = self.make_branch_and_tree('test')
+        tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
+        r1 = tree.commit('one')
+        r2 = tree.commit('two')
+        tree.branch.set_last_revision_info(2, r2)
+
+        reconciler = tree.branch.reconcile()
+        self.assertEquals([r1, r2], tree.branch.revision_history())

=== modified file 'bzrlib/tests/branch_implementations/test_sprout.py'
--- bzrlib/tests/branch_implementations/test_sprout.py	2009-05-06 05:36:28 +0000
+++ bzrlib/tests/branch_implementations/test_sprout.py	2009-05-28 16:04:39 +0000
@@ -155,6 +155,14 @@
         self.assertEqual(target,
                          tree.basis_tree().get_symlink_target('link-id'))
 
+    def test_sprout_with_ghost_in_mainline(self):
+        tree = self.make_branch_and_tree('tree1')
+        tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
+        tree.add('')
+        tree.commit('msg1', rev_id='rev1')
+        tree.commit('msg2', rev_id='rev2')
+        tree.bzrdir.sprout('target', revision_id='rev1')
+
     def assertBranchHookBranchIsStacked(self, pre_change_params):
         # Just calling will either succeed or fail.
         pre_change_params.branch.get_stacked_on_url()

=== modified file 'bzrlib/tests/per_repository/test_iter_reverse_revision_history.py'
--- bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2009-06-02 01:27:37 +0000
@@ -145,3 +145,23 @@
                                   repo, 'rev-2-4')
         self.assertRevHistoryList(['rev-2-5', 'rev-2-4', 'rev-2-3', 'rev-2-2',
                                    'rev-1-1'], repo, 'rev-2-5')
+
+    def test_ghost(self):
+        tree = self.make_branch_and_memory_tree('tree')
+        tree.lock_write()
+        try:
+            tree.add('')
+            tree.set_parent_ids(['spooky'], allow_leftmost_as_ghost=True)
+            tree.commit('1', rev_id='rev1')
+            tree.commit('2', rev_id='rev2')
+        finally:
+            tree.unlock()
+        iter = tree.branch.repository.iter_reverse_revision_history('rev2')
+        tree.branch.repository.lock_read()
+        try:
+            self.assertEquals('rev2', iter.next())
+            self.assertEquals('rev1', iter.next())
+            self.assertRaises(errors.RevisionNotPresent, iter.next)
+        finally:
+            tree.branch.repository.unlock()
+

=== modified file 'bzrlib/tests/per_repository/test_reconcile.py'
--- bzrlib/tests/per_repository/test_reconcile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_reconcile.py	2009-05-28 15:05:58 +0000
@@ -264,6 +264,40 @@
         self.assertEqual([None, 'the_ghost', 'ghost'], repo.get_ancestry('ghost'))
         self.assertEqual([None, 'the_ghost'], repo.get_ancestry('the_ghost'))
 
+    def test_text_from_ghost_revision(self):
+        repo = self.make_repository('text-from-ghost')
+        inv = Inventory(revision_id='final-revid')
+        inv.root.revision = 'root-revid'
+        ie = inv.add_path('bla', 'file', 'myfileid')
+        ie.revision = 'ghostrevid'
+        ie.text_size = 42
+        ie.text_sha1 = "bee68c8acd989f5f1765b4660695275948bf5c00"
+        rev = bzrlib.revision.Revision(timestamp=0,
+                                       timezone=None,
+                                       committer="Foo Bar <foo@example.com>",
+                                       message="Message",
+                                       revision_id='final-revid')
+        repo.lock_write()
+        try:
+            repo.start_write_group()
+            try:
+                repo.add_revision('final-revid', rev, inv)
+                try:
+                    repo.texts.add_lines(('myfileid', 'ghostrevid'),
+                        (('myfileid', 'ghost-text-parent'),),
+                        ["line1\n", "line2\n"])
+                except errors.RevisionNotPresent:
+                    raise TestSkipped("text ghost parents not supported")
+                if repo.supports_rich_root():
+                    root_id = inv.root.file_id
+                    repo.texts.add_lines((inv.root.file_id, inv.root.revision),
+                        [], [])
+            finally:
+                repo.commit_write_group()
+        finally:
+            repo.unlock()
+        repo.reconcile(thorough=True)
+
 
 class TestReconcileWithIncorrectRevisionCache(TestReconcile):
     """Ancestry data gets cached in knits and weaves should be reconcilable.

=== modified file 'bzrlib/tests/per_repository/test_revision.py'
--- bzrlib/tests/per_repository/test_revision.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_revision.py	2009-04-15 01:37:51 +0000
@@ -72,7 +72,7 @@
         tree1.commit(message="quux", allow_pointless=True, committer="jaq",
                      revprops={'empty':'',
                                'value':'one',
-                               'unicode':'\xb5',
+                               'unicode':u'\xb5',
                                'multiline':'foo\nbar\n\n'
                               })
         self.assertTrue(len(tree1.branch.revision_history()) > 0)

=== modified file 'bzrlib/tests/per_repository_reference/test_initialize.py'
--- bzrlib/tests/per_repository_reference/test_initialize.py	2009-05-29 09:56:12 +0000
+++ bzrlib/tests/per_repository_reference/test_initialize.py	2009-06-03 18:08:53 +0000
@@ -52,8 +52,4 @@
         trans = self.make_smart_server('stacked')
         repo = self.initialize_and_check_on_transport(base, trans)
         network_name = base.repository._format.network_name()
-        if network_name != repo._format.network_name():
-            raise tests.KnownFailure('Remote initialize_on_transport_ex()'
-                ' tries to "upgrade" the format because it doesn\'t have a'
-                ' branch format, and hard-codes the new repository format.')
         self.assertEqual(network_name, repo._format.network_name())

=== modified file 'bzrlib/tests/test__groupcompress.py'
--- bzrlib/tests/test__groupcompress.py	2009-04-29 05:53:21 +0000
+++ bzrlib/tests/test__groupcompress.py	2009-06-02 21:11:18 +0000
@@ -272,6 +272,25 @@
         di = self._gc_module.DeltaIndex('test text\n')
         self.assertEqual('DeltaIndex(1, 10)', repr(di))
 
+    def test_first_add_source_doesnt_index_until_make_delta(self):
+        di = self._gc_module.DeltaIndex()
+        self.assertFalse(di._has_index())
+        di.add_source(_text1, 0)
+        self.assertFalse(di._has_index())
+        # However, asking to make a delta will trigger the index to be
+        # generated, and will generate a proper delta
+        delta = di.make_delta(_text2)
+        self.assertTrue(di._has_index())
+        self.assertEqual('N\x90/\x1fdiffer from\nagainst other text\n', delta)
+
+    def test_second_add_source_triggers_make_index(self):
+        di = self._gc_module.DeltaIndex()
+        self.assertFalse(di._has_index())
+        di.add_source(_text1, 0)
+        self.assertFalse(di._has_index())
+        di.add_source(_text2, 0)
+        self.assertTrue(di._has_index())
+
     def test_make_delta(self):
         di = self._gc_module.DeltaIndex(_text1)
         delta = di.make_delta(_text2)

=== added file 'bzrlib/tests/test_bencode.py'
--- bzrlib/tests/test_bencode.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_bencode.py	2009-06-04 17:12:29 +0000
@@ -0,0 +1,235 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bencode structured encoding"""
+
+from bzrlib import tests
+
+def load_tests(standard_tests, module, loader):
+    # parameterize all tests in this module
+    suite = loader.suiteClass()
+    import bzrlib.util._bencode_py as py_module
+    scenarios = [('python', {'bencode': py_module})]
+    if CompiledBencodeFeature.available():
+        import bzrlib._bencode_pyx as c_module
+        scenarios.append(('C', {'bencode': c_module}))
+    else:
+        # the compiled module isn't available, so we add a failing test
+        class FailWithoutFeature(tests.TestCase):
+            def test_fail(self):
+                self.requireFeature(CompiledBencodeFeature)
+        suite.addTest(loader.loadTestsFromTestCase(FailWithoutFeature))
+    tests.multiply_tests(standard_tests, scenarios, suite)
+    return suite
+
+
+class _CompiledBencodeFeature(tests.Feature):
+
+    def _probe(self):
+        try:
+            import bzrlib._bencode_pyx
+        except ImportError:
+            return False
+        return True
+
+    def feature_name(self):
+        return 'bzrlib._bencode_pyx'
+
+CompiledBencodeFeature = _CompiledBencodeFeature()
+
+
+class TestBencodeDecode(tests.TestCase):
+
+    bencode = None
+
+    def _check(self, expected, source):
+        self.assertEquals(expected, self.bencode.bdecode(source))
+
+    def _run_check_error(self, exc, bad):
+        """Check that bdecoding a string raises a particular exception."""
+        self.assertRaises(exc, self.bencode.bdecode, bad)
+
+    def test_int(self):
+        self._check(0, 'i0e')
+        self._check(4, 'i4e')
+        self._check(123456789, 'i123456789e')
+        self._check(-10, 'i-10e')
+        self._check(int('1' * 1000), 'i' + ('1' * 1000) + 'e')
+
+    def test_long(self):
+        self._check(12345678901234567890L, 'i12345678901234567890e')
+        self._check(-12345678901234567890L, 'i-12345678901234567890e')
+
+    def test_malformed_int(self):
+        self._run_check_error(ValueError, 'ie')
+        self._run_check_error(ValueError, 'i-e')
+        self._run_check_error(ValueError, 'i-010e')
+        self._run_check_error(ValueError, 'i-0e')
+        self._run_check_error(ValueError, 'i00e')
+        self._run_check_error(ValueError, 'i01e')
+        self._run_check_error(ValueError, 'i-03e')
+        self._run_check_error(ValueError, 'i')
+        self._run_check_error(ValueError, 'i123')
+        self._run_check_error(ValueError, 'i341foo382e')
+
+    def test_string(self):
+        self._check('', '0:')
+        self._check('abc', '3:abc')
+        self._check('1234567890', '10:1234567890')
+
+    def test_large_string(self):
+        self.assertRaises(ValueError, self.bencode.bdecode, "2147483639:foo")
+
+    def test_malformed_string(self):
+        self._run_check_error(ValueError, '10:x')
+        self._run_check_error(ValueError, '10:')
+        self._run_check_error(ValueError, '10')
+        self._run_check_error(ValueError, '01:x')
+        self._run_check_error(ValueError, '00:')
+        self._run_check_error(ValueError, '35208734823ljdahflajhdf')
+        self._run_check_error(ValueError, '432432432432432:foo')
+        self._run_check_error(ValueError, ' 1:x') # leading whitespace
+        self._run_check_error(ValueError, '-1:x') # negative
+        self._run_check_error(ValueError, '1 x') # space vs colon
+        self._run_check_error(ValueError, '1x') # missing colon
+        self._run_check_error(ValueError, ('1' * 1000) + ':')
+
+    def test_list(self):
+        self._check([], 'le')
+        self._check(['', '', ''], 'l0:0:0:e')
+        self._check([1, 2, 3], 'li1ei2ei3ee')
+        self._check(['asd', 'xy'], 'l3:asd2:xye')
+        self._check([['Alice', 'Bob'], [2, 3]], 'll5:Alice3:Bobeli2ei3eee')
+
+    def test_list_deepnested(self):
+        self._run_check_error(RuntimeError, ("l" * 10000) + ("e" * 10000))
+
+    def test_malformed_list(self):
+        self._run_check_error(ValueError, 'l')
+        self._run_check_error(ValueError, 'l01:ae')
+        self._run_check_error(ValueError, 'l0:')
+        self._run_check_error(ValueError, 'li1e')
+        self._run_check_error(ValueError, 'l-3:e')
+
+    def test_dict(self):
+        self._check({}, 'de')
+        self._check({'':3}, 'd0:i3ee')
+        self._check({'age': 25, 'eyes': 'blue'}, 'd3:agei25e4:eyes4:bluee')
+        self._check({'spam.mp3': {'author': 'Alice', 'length': 100000}},
+                            'd8:spam.mp3d6:author5:Alice6:lengthi100000eee')
+
+    def test_dict_deepnested(self):
+        self._run_check_error(RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
+
+    def test_malformed_dict(self):
+        self._run_check_error(ValueError, 'd')
+        self._run_check_error(ValueError, 'defoobar')
+        self._run_check_error(ValueError, 'd3:fooe')
+        self._run_check_error(ValueError, 'di1e0:e')
+        self._run_check_error(ValueError, 'd1:b0:1:a0:e')
+        self._run_check_error(ValueError, 'd1:a0:1:a0:e')
+        self._run_check_error(ValueError, 'd0:0:')
+        self._run_check_error(ValueError, 'd0:')
+        self._run_check_error(ValueError, 'd432432432432432432:e')
+
+    def test_empty_string(self):
+        self.assertRaises(ValueError, self.bencode.bdecode, '')
+
+    def test_junk(self):
+        self._run_check_error(ValueError, 'i6easd')
+        self._run_check_error(ValueError, '2:abfdjslhfld')
+        self._run_check_error(ValueError, '0:0:')
+        self._run_check_error(ValueError, 'leanfdldjfh')
+
+    def test_unknown_object(self):
+        self.assertRaises(ValueError, self.bencode.bdecode, 'relwjhrlewjh')
+
+    def test_unsupported_type(self):
+        self._run_check_error(TypeError, float(1.5))
+        self._run_check_error(TypeError, None)
+        self._run_check_error(TypeError, lambda x: x)
+        self._run_check_error(TypeError, object)
+        self._run_check_error(TypeError, u"ie")
+
+    def test_decoder_type_error(self):
+        self.assertRaises(TypeError, self.bencode.bdecode, 1)
+
+
+class TestBencodeEncode(tests.TestCase):
+
+    bencode = None
+
+    def _check(self, expected, source):
+        self.assertEquals(expected, self.bencode.bencode(source))
+
+    def test_int(self):
+        self._check('i4e', 4)
+        self._check('i0e', 0)
+        self._check('i-10e', -10)
+
+    def test_long(self):
+        self._check('i12345678901234567890e', 12345678901234567890L)
+        self._check('i-12345678901234567890e', -12345678901234567890L)
+
+    def test_string(self):
+        self._check('0:', '')
+        self._check('3:abc', 'abc')
+        self._check('10:1234567890', '1234567890')
+
+    def test_list(self):
+        self._check('le', [])
+        self._check('li1ei2ei3ee', [1, 2, 3])
+        self._check('ll5:Alice3:Bobeli2ei3eee', [['Alice', 'Bob'], [2, 3]])
+
+    def test_list_as_tuple(self):
+        self._check('le', ())
+        self._check('li1ei2ei3ee', (1, 2, 3))
+        self._check('ll5:Alice3:Bobeli2ei3eee', (('Alice', 'Bob'), (2, 3)))
+
+    def test_list_deep_nested(self):
+        top = []
+        l = top
+        for i in range(10000):
+            l.append([])
+            l = l[0]
+        self.assertRaises(RuntimeError, self.bencode.bencode, 
+            top)
+
+    def test_dict(self):
+        self._check('de', {})
+        self._check('d3:agei25e4:eyes4:bluee', {'age': 25, 'eyes': 'blue'})
+        self._check('d8:spam.mp3d6:author5:Alice6:lengthi100000eee',
+                            {'spam.mp3': {'author': 'Alice',
+                                          'length': 100000}})
+
+    def test_dict_deep_nested(self):
+        d = top = {}
+        for i in range(10000):
+            d[''] = {}
+            d = d['']
+        self.assertRaises(RuntimeError, self.bencode.bencode, 
+            top)
+
+    def test_bencached(self):
+        self._check('i3e', self.bencode.Bencached(self.bencode.bencode(3)))
+
+    def test_invalid_dict(self):
+        self.assertRaises(TypeError, self.bencode.bencode, {1:"foo"})
+
+    def test_bool(self):
+        self._check('i1e', True)
+        self._check('i0e', False)
+

=== modified file 'bzrlib/tests/test_chk_map.py'
--- bzrlib/tests/test_chk_map.py	2009-05-13 21:59:57 +0000
+++ bzrlib/tests/test_chk_map.py	2009-06-05 18:03:40 +0000
@@ -1560,13 +1560,66 @@
         child.map(None, ("baz",), "val")
         node.add_node("b", child)
 
-        key_filter = (('foo',), ('fob',), ('bar',), ('baz',))
+        # Note that 'ram' doesn't match anything, so it should be freely
+        # ignored
+        key_filter = (('foo',), ('fob',), ('bar',), ('baz',), ('ram',))
         for child, node_key_filter in node._iter_nodes(None,
                                                        key_filter=key_filter):
-            # each child could matches two key filters, so make sure they were
+            # each child could match two key filters, so make sure they were
             # both included.
             self.assertEqual(2, len(node_key_filter))
 
+    def make_fo_fa_node(self):
+        node = InternalNode('f')
+        child = LeafNode()
+        child.set_maximum_size(100)
+        child.map(None, ("foo",), "val")
+        child.map(None, ("fob",), "val")
+        node.add_node('fo', child)
+        child = LeafNode()
+        child.set_maximum_size(100)
+        child.map(None, ("far",), "val")
+        child.map(None, ("faz",), "val")
+        node.add_node("fa", child)
+        return node
+
+    def test__iter_nodes_single_entry(self):
+        node = self.make_fo_fa_node()
+        key_filter = [('foo',)]
+        nodes = list(node._iter_nodes(None, key_filter=key_filter))
+        self.assertEqual(1, len(nodes))
+        self.assertEqual(key_filter, nodes[0][1])
+
+    def test__iter_nodes_single_entry_misses(self):
+        node = self.make_fo_fa_node()
+        key_filter = [('bar',)]
+        nodes = list(node._iter_nodes(None, key_filter=key_filter))
+        self.assertEqual(0, len(nodes))
+
+    def test__iter_nodes_mixed_key_width(self):
+        node = self.make_fo_fa_node()
+        key_filter = [('foo', 'bar'), ('foo',), ('fo',), ('b',)]
+        nodes = list(node._iter_nodes(None, key_filter=key_filter))
+        self.assertEqual(1, len(nodes))
+        matches = key_filter[:]
+        matches.remove(('b',))
+        self.assertEqual(sorted(matches), sorted(nodes[0][1]))
+
+    def test__iter_nodes_match_all(self):
+        node = self.make_fo_fa_node()
+        key_filter = [('foo', 'bar'), ('foo',), ('fo',), ('f',)]
+        nodes = list(node._iter_nodes(None, key_filter=key_filter))
+        self.assertEqual(2, len(nodes))
+
+    def test__iter_nodes_fixed_widths_and_misses(self):
+        node = self.make_fo_fa_node()
+        # foo and faa should both match one child, baz should miss
+        key_filter = [('foo',), ('faa',), ('baz',)]
+        nodes = list(node._iter_nodes(None, key_filter=key_filter))
+        self.assertEqual(2, len(nodes))
+        for node, matches in nodes:
+            self.assertEqual(1, len(matches))
+
     def test_iteritems_empty_new(self):
         node = InternalNode()
         self.assertEqual([], sorted(node.iteritems(None)))

=== added file 'bzrlib/tests/test_chk_serializer.py'
--- bzrlib/tests/test_chk_serializer.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_chk_serializer.py	2009-06-04 21:50:04 +0000
@@ -0,0 +1,116 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from cStringIO import StringIO
+
+from bzrlib import (
+    errors,
+    revision,
+    serializer,
+    )
+from bzrlib.chk_serializer import (
+    chk_bencode_serializer,
+    )
+from bzrlib.revision import (
+    Revision,
+    )
+from bzrlib.tests import TestCase
+
+_working_revision_bencode1 = ('l'
+    'l6:formati10ee'
+    'l9:committer54:Canonical.com Patch Queue Manager <pqm@pqm.ubuntu.com>e'
+    'l8:timezonei3600ee'
+    'l10:propertiesd11:branch-nick6:+trunkee'
+    'l9:timestamp14:1242300770.844e'
+    'l11:revision-id50:pqm@pqm.ubuntu.com-20090514113250-jntkkpminfn3e0tze'
+    'l10:parent-ids'
+        'l'
+        '50:pqm@pqm.ubuntu.com-20090514104039-kggemn7lrretzpvc'
+        '48:jelmer@samba.org-20090510012654-jp9ufxquekaokbeo'
+        'ee'
+    'l14:inventory-sha140:4a2c7fb50e077699242cf6eb16a61779c7b680a7e'
+    'l7:message35:(Jelmer) Move dpush to InterBranch.e'
+    'e')
+
+_working_revision_bencode1_no_timezone = ('l'
+    'l6:formati10ee'
+    'l9:committer54:Canonical.com Patch Queue Manager <pqm@pqm.ubuntu.com>e'
+    'l9:timestamp14:1242300770.844e'
+    'l10:propertiesd11:branch-nick6:+trunkee'
+    'l11:revision-id50:pqm@pqm.ubuntu.com-20090514113250-jntkkpminfn3e0tze'
+    'l10:parent-ids'
+        'l'
+        '50:pqm@pqm.ubuntu.com-20090514104039-kggemn7lrretzpvc'
+        '48:jelmer@samba.org-20090510012654-jp9ufxquekaokbeo'
+        'ee'
+    'l14:inventory-sha140:4a2c7fb50e077699242cf6eb16a61779c7b680a7e'
+    'l7:message35:(Jelmer) Move dpush to InterBranch.e'
+    'e')
+
+
+class TestBEncodeSerializer1(TestCase):
+    """Test BEncode serialization"""
+
+    def test_unpack_revision(self):
+        """Test unpacking a revision"""
+        rev = chk_bencode_serializer.read_revision_from_string(
+                _working_revision_bencode1)
+        self.assertEquals(rev.committer,
+           "Canonical.com Patch Queue Manager <pqm@pqm.ubuntu.com>")
+        self.assertEquals(rev.inventory_sha1,
+           "4a2c7fb50e077699242cf6eb16a61779c7b680a7")
+        self.assertEquals(["pqm@pqm.ubuntu.com-20090514104039-kggemn7lrretzpvc",
+            "jelmer@samba.org-20090510012654-jp9ufxquekaokbeo"],
+            rev.parent_ids)
+        self.assertEquals("(Jelmer) Move dpush to InterBranch.", rev.message)
+        self.assertEquals("pqm@pqm.ubuntu.com-20090514113250-jntkkpminfn3e0tz",
+           rev.revision_id)
+        self.assertEquals({"branch-nick": u"+trunk"}, rev.properties)
+        self.assertEquals(3600, rev.timezone)
+
+    def test_written_form_matches(self):
+        rev = chk_bencode_serializer.read_revision_from_string(
+                _working_revision_bencode1)
+        as_str = chk_bencode_serializer.write_revision_to_string(rev)
+        self.assertEqualDiff(_working_revision_bencode1, as_str)
+
+    def test_unpack_revision_no_timezone(self):
+        rev = chk_bencode_serializer.read_revision_from_string(
+            _working_revision_bencode1_no_timezone)
+        self.assertEquals(None, rev.timezone)
+
+    def assertRoundTrips(self, serializer, orig_rev):
+        text = serializer.write_revision_to_string(orig_rev)
+        new_rev = serializer.read_revision_from_string(text)
+        self.assertEquals(orig_rev, new_rev)
+
+    def test_roundtrips_non_ascii(self):
+        rev = Revision("revid1")
+        rev.message = u"\n\xe5me"
+        rev.committer = u'Erik B\xe5gfors'
+        rev.timestamp = 1242385452
+        rev.inventory_sha1 = "4a2c7fb50e077699242cf6eb16a61779c7b680a7"
+        rev.timezone = 3600
+        self.assertRoundTrips(chk_bencode_serializer, rev)
+
+    def test_roundtrips_xml_invalid_chars(self):
+        rev = Revision("revid1")
+        rev.message = "\t\ue000"
+        rev.committer = u'Erik B\xe5gfors'
+        rev.timestamp = 1242385452
+        rev.timezone = 3600
+        rev.inventory_sha1 = "4a2c7fb50e077699242cf6eb16a61779c7b680a7"
+        self.assertRoundTrips(chk_bencode_serializer, rev)

=== modified file 'bzrlib/tests/test_mail_client.py'
--- bzrlib/tests/test_mail_client.py	2009-05-11 18:35:20 +0000
+++ bzrlib/tests/test_mail_client.py	2009-06-04 19:56:42 +0000
@@ -191,9 +191,10 @@
     def test_commandline(self):
         claws = mail_client.Claws(None)
         commandline = claws._get_compose_commandline(
-            None, None, 'file%')
+            'jrandom@example.org', None, 'file%')
         self.assertEqual(
-            ['--compose', 'mailto:?', '--attach', 'file%'], commandline)
+            ['--compose', 'mailto:jrandom@example.org?', '--attach', 'file%'],
+            commandline)
         commandline = claws._get_compose_commandline(
             'jrandom@example.org', 'Hi there!', None)
         self.assertEqual(
@@ -217,6 +218,30 @@
             self.assertFalse(isinstance(item, unicode),
                 'Command-line item %r is unicode!' % item)
 
+    def test_with_from(self):
+        claws = mail_client.Claws(None)
+        cmdline = claws._get_compose_commandline(
+            u'jrandom@example.org', None, None, None, u'qrandom@example.com')
+        self.assertEqual(
+            ['--compose',
+             'mailto:jrandom@example.org?from=qrandom%40example.com'],
+            cmdline)
+
+    def test_to_required(self):
+        claws = mail_client.Claws(None)
+        self.assertRaises(errors.NoMailAddressSpecified,
+                          claws._get_compose_commandline,
+                          None, None, 'file%')
+
+    def test_with_body(self):
+        claws = mail_client.Claws(None)
+        cmdline = claws._get_compose_commandline(
+            u'jrandom@example.org', None, None, 'This is some body text')
+        self.assertEqual(
+            ['--compose',
+             'mailto:jrandom@example.org?body=This%20is%20some%20body%20text'],
+            cmdline)
+
 
 class TestEditor(tests.TestCase):
 

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2009-05-23 04:55:52 +0000
+++ bzrlib/tests/test_osutils.py	2009-06-04 11:05:37 +0000
@@ -1760,3 +1760,10 @@
     def test_os_readlink_link_decoding(self):
         self.assertEquals(self.target.encode(osutils._fs_enc),
                           os.readlink(self.link.encode(osutils._fs_enc)))
+
+
+class TestConcurrency(tests.TestCase):
+
+    def test_local_concurrency(self):
+        concurrency = osutils.local_concurrency()
+        self.assertIsInstance(concurrency, int)

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2009-05-29 09:59:25 +0000
+++ bzrlib/tests/test_repository.py	2009-06-03 01:37:27 +0000
@@ -48,8 +48,8 @@
     get_transport,
     )
 from bzrlib.transport.memory import MemoryServer
-from bzrlib.util import bencode
 from bzrlib import (
+    bencode,
     bzrdir,
     errors,
     inventory,

=== modified file 'bzrlib/tests/test_serializer.py'
--- bzrlib/tests/test_serializer.py	2009-04-03 21:50:40 +0000
+++ bzrlib/tests/test_serializer.py	2009-06-03 20:02:46 +0000
@@ -19,6 +19,7 @@
 
 
 from bzrlib import (
+    chk_serializer,
     serializer,
     xml4,
     xml5,
@@ -43,3 +44,5 @@
                       serializer.format_registry.get('7'))
         self.assertIs(xml8.serializer_v8,
                       serializer.format_registry.get('8'))
+        self.assertIs(chk_serializer.chk_serializer_255_bigpage,
+                      serializer.format_registry.get('9'))

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2009-05-11 10:25:32 +0000
+++ bzrlib/tests/test_smart.py	2009-05-26 09:20:17 +0000
@@ -29,6 +29,7 @@
 import tarfile
 
 from bzrlib import (
+    bencode,
     bzrdir,
     errors,
     pack,
@@ -51,7 +52,6 @@
     split_suite_by_re,
     )
 from bzrlib.transport import chroot, get_transport
-from bzrlib.util import bencode
 
 
 def load_tests(standard_tests, module, loader):

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2009-03-28 09:57:23 +0000
+++ bzrlib/tests/test_source.py	2009-05-27 08:33:37 +0000
@@ -42,9 +42,9 @@
 
 # Files which are listed here will be skipped when testing for Copyright (or
 # GPL) statements.
-COPYRIGHT_EXCEPTIONS = ['bzrlib/lsprof.py']
+COPYRIGHT_EXCEPTIONS = ['bzrlib/lsprof.py', 'bzrlib/_bencode_py.py']
 
-LICENSE_EXCEPTIONS = ['bzrlib/lsprof.py']
+LICENSE_EXCEPTIONS = ['bzrlib/lsprof.py', 'bzrlib/_bencode_py.py']
 # Technically, 'bzrlib/lsprof.py' should be 'bzrlib/util/lsprof.py',
 # (we do not check bzrlib/util/, since that is code bundled from elsewhere)
 # but for compatibility with previous releases, we don't want to move it.

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2009-05-23 04:55:52 +0000
+++ bzrlib/tests/test_transform.py	2009-05-26 09:20:17 +0000
@@ -20,6 +20,7 @@
 import sys
 
 from bzrlib import (
+    bencode,
     errors,
     generate_ids,
     osutils,
@@ -53,7 +54,6 @@
                               build_tree, get_backup_name,
                               _FileMover, resolve_checkout,
                               TransformPreview, create_from_tree)
-from bzrlib.util import bencode
 
 
 class TestTreeTransform(tests.TestCaseWithTransport):

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2009-05-28 14:25:00 +0000
+++ bzrlib/transform.py	2009-06-03 01:37:27 +0000
@@ -22,6 +22,7 @@
 lazy_import(globals(), """
 from bzrlib import (
     annotate,
+    bencode,
     bzrdir,
     delta,
     errors,
@@ -30,7 +31,6 @@
     osutils,
     revision as _mod_revision,
     )
-from bzrlib.util import bencode
 """)
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, NotVersionedError, CantMoveRoot,

=== renamed file 'bzrlib/util/bencode.py' => 'bzrlib/util/_bencode_py.py'
--- bzrlib/util/bencode.py	2009-02-23 15:29:35 +0000
+++ bzrlib/util/_bencode_py.py	2009-06-03 14:14:31 +0000
@@ -86,6 +86,8 @@
         return (r, f + 1)
 
     def bdecode(self, x):
+        if type(x) != str:
+            raise TypeError
         try:
             r, l = self.decode_func[x[0]](x, 0)
         except (IndexError, KeyError):

=== removed file 'bzrlib/util/tests/test_bencode.py'
--- bzrlib/util/tests/test_bencode.py	2009-02-23 15:29:35 +0000
+++ bzrlib/util/tests/test_bencode.py	1970-01-01 00:00:00 +0000
@@ -1,234 +0,0 @@
-# bencode structured encoding
-#
-# Written by Petru Paler
-#
-# Permission is hereby granted, free of charge, to any person
-# obtaining a copy of this software and associated documentation files
-# (the "Software"), to deal in the Software without restriction,
-# including without limitation the rights to use, copy, modify, merge,
-# publish, distribute, sublicense, and/or sell copies of the Software,
-# and to permit persons to whom the Software is furnished to do so,
-# subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be
-# included in all copies or substantial portions of the Software.
-#
-# The Software is provided "AS IS", without warranty of any kind,
-# express or implied, including but not limited to the warranties of
-# merchantability,  fitness for a particular purpose and
-# noninfringement. In no event shall the  authors or copyright holders
-# be liable for any claim, damages or other liability, whether in an
-# action of contract, tort or otherwise, arising from, out of or in
-# connection with the Software or the use or other dealings in the
-# Software.
-
-
-from bzrlib.util.bencode import bencode, bdecode, bdecode_as_tuple, Bencached
-from bzrlib.tests import TestCase
-
-class TestBencode(TestCase):
-    # tests moved from within the bencode module so they're not run on every
-    # startup
-
-    def test_bdecode(self):
-        try:
-            bdecode('0:0:')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('ie')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('i341foo382e')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode('i4e') == 4L
-        assert bdecode('i0e') == 0L
-        assert bdecode('i123456789e') == 123456789L
-        assert bdecode('i-10e') == -10L
-        try:
-            bdecode('i-0e')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('i123')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('i6easd')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('35208734823ljdahflajhdf')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('2:abfdjslhfld')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode('0:') == ''
-        assert bdecode('3:abc') == 'abc'
-        assert bdecode('10:1234567890') == '1234567890'
-        try:
-            bdecode('02:xy')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('l')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode('le') == []
-        try:
-            bdecode('leanfdldjfh')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode('l0:0:0:e') == ['', '', '']
-        try:
-            bdecode('relwjhrlewjh')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode('li1ei2ei3ee') == [1, 2, 3]
-        assert bdecode('l3:asd2:xye') == ['asd', 'xy']
-        assert bdecode('ll5:Alice3:Bobeli2ei3eee') == [['Alice', 'Bob'], [2, 3]]
-        try:
-            bdecode('d')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('defoobar')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode('de') == {}
-        assert bdecode('d3:agei25e4:eyes4:bluee') == {'age': 25, 'eyes': 'blue'}
-        assert bdecode('d8:spam.mp3d6:author5:Alice6:lengthi100000eee') == {'spam.mp3': {'author': 'Alice', 'length': 100000}}
-        try:
-            bdecode('d3:fooe')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('di1e0:e')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('d1:b0:1:a0:e')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('d1:a0:1:a0:e')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('i03e')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('l01:ae')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('9999:x')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('l0:')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('d0:0:')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('d0:')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('00:')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('l-3:e')
-            assert 0
-        except ValueError:
-            pass
-        try:
-            bdecode('i-03e')
-            assert 0
-        except ValueError:
-            pass
-        bdecode('d0:i3ee')
-
-
-    def test_bdecode_as_tuple(self):
-        assert bdecode_as_tuple('le') == ()
-        try:
-            bdecode_as_tuple('leanfdldjfh')
-            assert 0
-        except ValueError:
-            pass
-        assert bdecode_as_tuple('l0:0:0:e') == ('', '', '')
-        assert bdecode_as_tuple('li1ei2ei3ee') == (1, 2, 3)
-        assert bdecode_as_tuple('l3:asd2:xye') == ('asd', 'xy')
-        assert bdecode_as_tuple('ll5:Alice3:Bobeli2ei3eee') == (('Alice', 'Bob'),
-                (2, 3))
-        try:
-            bdecode_as_tuple('l-3:e')
-            assert 0
-        except ValueError:
-            pass
-
-
-    def test_bencode(self):
-        assert bencode(4) == 'i4e'
-        assert bencode(0) == 'i0e'
-        assert bencode(-10) == 'i-10e'
-        assert bencode(12345678901234567890L) == 'i12345678901234567890e'
-        assert bencode('') == '0:'
-        assert bencode('abc') == '3:abc'
-        assert bencode('1234567890') == '10:1234567890'
-        assert bencode([]) == 'le'
-        assert bencode([1, 2, 3]) == 'li1ei2ei3ee'
-        assert bencode([['Alice', 'Bob'], [2, 3]]) == 'll5:Alice3:Bobeli2ei3eee'
-        assert bencode({}) == 'de'
-        assert bencode({'age': 25, 'eyes': 'blue'}) == 'd3:agei25e4:eyes4:bluee'
-        assert bencode({'spam.mp3': {'author': 'Alice', 'length': 100000}}) == 'd8:spam.mp3d6:author5:Alice6:lengthi100000eee'
-        assert bencode(Bencached(bencode(3))) == 'i3e'
-        assert bencode(True) == 'i1e'
-        assert bencode(False) == 'i0e'
-        try:
-            bencode({1: 'foo'})
-            assert 0
-        except TypeError:
-            pass
-
-

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2009-04-29 17:02:36 +0000
+++ bzrlib/versionedfile.py	2009-05-26 09:20:17 +0000
@@ -47,7 +47,7 @@
 from bzrlib.registry import Registry
 from bzrlib.symbol_versioning import *
 from bzrlib.textmerge import TextMerge
-from bzrlib.util import bencode
+from bzrlib import bencode
 
 
 adapter_registry = Registry()

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2009-05-21 12:16:23 +0000
+++ bzrlib/workingtree.py	2009-06-05 19:57:15 +0000
@@ -451,7 +451,7 @@
             path = self.id2path(file_id)
         file_obj = self.get_file_byname(path, filtered=False)
         stat_value = _fstat(file_obj.fileno())
-        if self.supports_content_filtering() and filtered:
+        if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(path)
             file_obj = filtered_input_file(file_obj, filters)
         return (file_obj, stat_value)
@@ -462,7 +462,7 @@
     def get_file_byname(self, filename, filtered=True):
         path = self.abspath(filename)
         f = file(path, 'rb')
-        if self.supports_content_filtering() and filtered:
+        if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(filename)
             return filtered_input_file(f, filters)
         else:

=== modified file 'setup.py'
--- setup.py	2009-05-23 04:55:52 +0000
+++ setup.py	2009-06-03 14:14:31 +0000
@@ -259,13 +259,14 @@
         define_macros=define_macros, libraries=libraries))
 
 
+add_pyrex_extension('bzrlib._bencode_pyx')
 add_pyrex_extension('bzrlib._btree_serializer_c')
+add_pyrex_extension('bzrlib._chk_map_pyx', libraries=['z'])
+add_pyrex_extension('bzrlib._chunks_to_lines_pyx')
 add_pyrex_extension('bzrlib._groupcompress_pyx',
                     extra_source=['bzrlib/diff-delta.c'])
-add_pyrex_extension('bzrlib._chunks_to_lines_pyx')
 add_pyrex_extension('bzrlib._knit_load_data_c')
 add_pyrex_extension('bzrlib._rio_pyx')
-add_pyrex_extension('bzrlib._chk_map_pyx', libraries=['z'])
 if sys.platform == 'win32':
     add_pyrex_extension('bzrlib._dirstate_helpers_c',
                         libraries=['Ws2_32'])

